/*
 *  Copyright (C) 1999,2000,2001,2002,2003,2005 Free Software Foundation, Inc.
 *  Copyright (C) 2005 Timothy Baldwin.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


	GRUB_MEMORY_MACHINE_CR0_PE_ON = 1

	.global	_start
_start:

start:	jmp	start

multiboot_header:
#	.long	0x1BADB002
	.long	0x00100000
	.long	0 - 0x00100000 - 0x1BADB002
	.long	multiboot_header
	.long	start
	.long	start + 0x8000
	.long	start + 0x10000
	.long	mstart


	.code32
mstart: movl	$start, %edi
	movl	$0x00100000, %esi
	movl	$0x8000, %ecx
	cld
	rep movsl
	cli
	lgdt	gdtdesc
	ljmp	$8, $exit_pmode

	.code16

stack:	.long	0x7000
	.word	0x10

	.p2align	2	/* force 4-byte alignment */
gdt:
	.word	0, 0
	.byte	0, 0, 0, 0

	/* -- 16 bit real mode CS --
	 * base = 0x00000000, limit 0x0FFFF (1 B Granularity), present
	 * type = 16 bit code execute/read only/conforming, DPL = 0
	 */
	.word	0xFFFF, 0
	.byte	0, 0x9E, 0, 0

	/* -- 16 bit real mode DS --
	 * base = 0x00000000, limit 0x0FFFF (1 B Granularity), present
	 * type = 16 bit data read/write, DPL = 0
	 */
	.word	0xFFFF, 0
	.byte	0, 0x92, 0, 0

	/* -- data segment --
	 * base = 0x00000000, limit 0xFFFFF (4 KiB Granularity), present
	 * type = 32 bit data read/write, DPL = 0 
	 */
	.word	0xFFFF, 0
	.byte	0, 0x92, 0xCF, 0

/* this is the GDT descriptor */
gdtdesc:
	.word	0x1F			/* limit */
	.long	gdt			/* addr */

lstart: 

	xorw    %ax, %ax
        movw    %ax, %es
	xorw	%si, %si
	movl	$start, %edi
	movw	$(512 * 5), %cx
	cld
	rep	movsb


	cli
	data32	addr32	lgdt	gdtdesc - start
	movl	%cr0, %eax
	orl 	$GRUB_MEMORY_MACHINE_CR0_PE_ON, %eax
	movl	%eax, %cr0
	data32	ljmp	$8, $linux_pmode

linux_pmode:
	movw	$0x18, %ax
	movw	%ax, %ds
	movw	%ax, %es
	movl	$0x100000, %esi
	movl	$0x30000, %ecx
	addr32 rep movsb

exit_pmode:
	movw	$0x10, %ax
	movw	%ax, %ds
	movw	%ax, %es
	movw	%ax, %fs
	movw	%ax, %gs
	lss	stack, %esp
	xorw	%dx, %dx # Set drive referenece
	movl	%cr0, %eax
	andl 	$(~GRUB_MEMORY_MACHINE_CR0_PE_ON), %eax
#1:	jmp	1b
	movl	%eax, %cr0
	ljmp	$0, $0x8200


	. = start + 0x1E0



	. = start + 0x1F1
setup_sects:	.byte	4
	. = start + 0x1FE
boot_flag:	.word	0xAA55
		jmp     lstart1
		.ascii	"HdrS"
		.word	0x0203
	. = start + 0x211
loadflags:	.byte	1
	. = start + 0x214
code32_start:	.long	0x100000
	. = start + 0x230

lstart1:

	pushw	%ds
	pushw	$lstart - start
	lret

	.if	0
print:  pusha
print.loop: lodsb
        cmpb    $0,%al
        jz      print.exit
        movb    $0xE,%ah
        movw    $7,%bx
        int     $0x10
        jmp     print.loop
print.exit: popa
        ret



halt:	hlt
	jmp	halt

started:.asciz	"started"

	.endif

	.p2align 2
end: 

. = start + 0x8000 - 0x7C00 -1
	.byte	0
